import * as runnerApi from "../proto/beam_runner_api";
import { Reader, Writer } from "protobufjs";
import { Coder, Context, ProtoContext } from "./coders";
import { Window, GlobalWindow, Instant, KV, PaneInfo, WindowedValue } from "../values";
/**
 * @fileoverview Defines all of the Apache Beam required coders.
 *
 * These are the coders necessary for encoding the data types required by
 * the Apache Beam model. They provide standardized ways of encode data for
 * communication between the runner, the Beam workers, and the user's code.
 * For example for any aggregations the runner and the SDK need to agree on
 * the encoding of key-value pairs; so that the SDK will encode keys properly,
 * and the runner will be able to group elements of the
 * same key together.
 *
 * The formal specifications for these coders can be found in
 * model/pipeline/src/main/proto/beam_runner_api.proto
 */
/**
 * Coder for byte-array data types.
 */
export declare class BytesCoder implements Coder<Uint8Array> {
    static URN: string;
    static INSTANCE: BytesCoder;
    type: string;
    /**
     * Encode the input element (a byte-string) into the output byte stream from `writer`.
     * If context is `needsDelimiters`, the byte string is encoded prefixed with a
     * varint representing its length.
     *
     * If the context is `wholeStream`, the byte string is encoded as-is.
     *
     * For example:
     * ```js
     * const w1 = new Writer()
     * const data = new TextEncoder().encode("bytes")
     * new BytesCoder().encode(data, w1, Context.needsDelimiters)
     * console.log(w1.finish())  // ==> prints Uint8Array(6) [ 5, 98, 121, 116, 101, 115 ], where 5 is the length prefix.
     * const w2 = new Writer()
     * new BytesCoder().encode("bytes", w1, Context.wholeStream)
     * console.log(w2.finish())  // ==> prints Uint8Array(5) [ 98, 121, 116, 101, 115 ], without the length prefix
     * ```
     * @param value - a byte array to encode. This represents an element to be encoded.
     * @param writer - a writer to access the stream of bytes with encoded data
     * @param context - whether to encode the data with delimiters (`Context.needsDelimiters`), or without (`Context.wholeStream`).
     */
    encode(value: Uint8Array, writer: Writer, context: Context): void;
    /**
     * Decode the input byte stream into a byte array.
     * If context is `needsDelimiters`, the first bytes will be interpreted as a var-int32 encoding
     * the length of the data.
     *
     * If the context is `wholeStream`, the whole input stream is decoded as-is.
     *
     * @param reader - a reader to access the input byte stream
     * @param context - whether the data is encoded with delimiters (`Context.needsDelimiters`), or without (`Context.wholeStream`).
     * @returns
     */
    decode(reader: Reader, context: Context): Uint8Array;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
/**
 * A coder for a key-value pair.
 */
export declare class KVCoder<K, V> implements Coder<KV<K, V>> {
    static URN: string;
    type: string;
    keyCoder: Coder<K>;
    valueCoder: Coder<V>;
    constructor(keyCoder: Coder<K>, valueCoder: Coder<V>);
    /**
     * Encode the input element (a key-value pair) into a byte output stream. They key and value are encoded one after the
     * other (first key, then value). The key is encoded with `Context.needsDelimiters`, while the value is encoded with
     * the input context of the `KVCoder`.
     *
     * For example:
     * ```js
     * let w1 = new Writer()
     * let data = {'key': 'k1', 'value': 'v1'}
     * new KVCoder(new StrUtf8Coder(), new StrUtf8Coder()).encode(data, w1, Context.needsDelimiters)
     * console.log(w1.finish())  // ==> prints  Uint8Array(6) [ 2, 107, 49, 2, 118, 49 ]
     * // Note that bytes with value 2 represent length prefixes, and other bytes represent the encoded strings.
     * ```
     * @param element - the element to be encoded into an output byte stream
     * @param writer - the writer that interfaces with the output byte stream
     * @param context - whether to encode the data with delimiters (`Context.needsDelimiters`), or without (`Context.wholeStream`).
     */
    encode(element: KV<K, V>, writer: Writer, context: Context): void;
    /**
     * Decode the input byte stream into a `KV` element.
     * @param reader
     * @param context
     * @returns the decoded element into a `KV`-typed object.
     */
    decode(reader: Reader, context: Context): KV<K, V>;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
/**
 * A coder for a 'list' or a series of elements of the same type.
 */
export declare class IterableCoder<T> implements Coder<Iterable<T>> {
    static URN: string;
    type: string;
    elementCoder: Coder<T>;
    constructor(elementCoder: Coder<T>);
    /**
     * Encode the input iterable into a byte output stream. Elements can be encoded in two different ways:
     *
     * - If the length of the input iterable is known a-priori, then the length is encoded with a 32-bit
     *     fixed-length integer.
     * - If the length of the input iterable is not known a-priori, then a 32-bit integer with a value
     *     of `-1` is encoded in the first position (instead of the length), and
     *
     * Then, each element is encoded individually in `Context.needsDelimiters`.
     *
     * For example:
     * ```js
     * let w1 = new Writer()
     * let data = ["a", "b", "c"]
     * new IterableCoder(new StrUtf8Coder()).encode(data, w1, Context.needsDelimiters)
     * console.log(w1.finish())  // ==> prints
     * // Uint8Array(10) [
     * //    0, 0,  0, 3,  1,
     * //    97, 1, 98, 1, 99
     * // ]
     * // The first 4 bytes represent the 32-bit-encoded length of the iterable (0,0,0,3),
     * // and the next 6 bytes represent each of the elements in the iterable with their length
     * // encoded as a prefixed var int (1), and their values afterwards (97, 98, 99).
     * let w2 = new Writer()
     * let data = ["a", "b", "c"]
     * let encodeIterable = function* gen() {yield *data}
     * new IterableCoder(new StrUtf8Coder()).encode(encodeIterable, w2, Context.needsDelimiters)
     * console.log(w2.finish())  // ==> prints
     * // Uint8Array(11) [
     * //    255, 255, 255, 255,
     * //    3, 1, 97, 1, 98,
     * //    1, 99, 0,
     * // ]
     * // The first 4 bytes represent the 32-bit-encoded -1 to represent unknown length of the
     * //  iterable, the next byte represents the length of the upcoming batch (3),
     * //  and the next 6 bytes represent each of the elements in the iterable with their length
     * // encoded as a prefixed var int (1), and their values afterwards (97, 98, 99).
     * // Finally, the last byte tags the end of the iterable as a zero (0)
     * ```
     */
    encode(element: Iterable<T>, writer: Writer, context: Context): void;
    _flushBatch(batch: Array<T>, writer: Writer): void;
    decode(reader: Reader, context: Context): Iterable<T>;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
export declare class LengthPrefixedCoder<T> implements Coder<T> {
    static URN: string;
    elementCoder: Coder<T>;
    constructor(elementCoder: Coder<T>);
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
    encode(element: T, writer: Writer, context: Context): void;
    decode(reader: Reader, context: Context): T;
}
export declare class FullWindowedValueCoder<T, W extends Window> implements Coder<WindowedValue<T>> {
    elementCoder: Coder<T>;
    windowCoder: Coder<W>;
    static URN: string;
    windowIterableCoder: IterableCoder<W>;
    constructor(elementCoder: Coder<T>, windowCoder: Coder<W>);
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
    encode(windowedValue: WindowedValue<T>, writer: Writer, context: Context): void;
    decode(reader: Reader, context: Context): WindowedValue<T>;
}
export declare class GlobalWindowCoder implements Coder<GlobalWindow> {
    static URN: string;
    static INSTANCE: GlobalWindowCoder;
    encode(value: GlobalWindow, writer: Writer, context: Context): void;
    decode(reader: Reader, context: Context): GlobalWindow;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
export declare class InstantCoder implements Coder<Instant> {
    static INSTANCE: InstantCoder;
    static INSTANT_BYTES: number;
    decode(reader: Reader, context: Context): Instant;
    encode(element: Instant, writer: Writer, context: Context): void;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
export declare class PaneInfoCoder implements Coder<PaneInfo> {
    static INSTANCE: PaneInfoCoder;
    static ONE_AND_ONLY_FIRING: PaneInfo;
    private static decodeTiming;
    private static encodeTiming;
    private static chooseEncoding;
    decode(reader: Reader, context: Context): PaneInfo;
    encode(value: PaneInfo, writer: Writer, context: Context): void;
    toProto(pipelineContext: ProtoContext): runnerApi.Coder;
}
