import { Pipeline } from "./internal/pipeline";
import { PTransform, AsyncPTransform } from "./transforms/transform";
/**
 * The base object on which one can start building a Beam DAG.
 * Generally followed by a source-like transform such as a read or impulse.
 */
export declare class Root {
    /** @internal */
    pipeline: Pipeline;
    /** @internal */
    constructor(pipeline: Pipeline);
    apply<OutputT extends PValue<any>>(transform: PTransform<Root, OutputT>): OutputT;
    applyAsync<OutputT extends PValue<any>>(transform: AsyncPTransform<Root, OutputT>): Promise<OutputT>;
}
/**
 * A deferred, possibly distributed collection of elements.
 * See https://beam.apache.org/documentation/programming-guide/#pcollections
 */
export declare class PCollection<T> {
    /** @internal */
    pipeline: Pipeline;
    private id;
    private computeId;
    /** @internal */
    constructor(pipeline: Pipeline, id: string | (() => string));
    /** @internal */
    getId(): string;
    apply<OutputT extends PValue<any>>(transform: PTransform<PCollection<T>, OutputT>): any;
    applyAsync<OutputT extends PValue<any>>(transform: AsyncPTransform<PCollection<T>, OutputT>): any;
    map<OutputT, ContextT>(fn: (ContextT extends undefined ? (element: T) => OutputT : never) | ((element: T, context: ContextT) => OutputT), context?: ContextT): PCollection<OutputT>;
    flatMap<OutputT, ContextT>(fn: (ContextT extends undefined ? (element: T) => Iterable<OutputT> : never) | ((element: T, context: ContextT) => Iterable<OutputT>), context?: ContextT): PCollection<OutputT>;
    root(): Root;
}
/**
 * The type of object that may be consumed or produced by a PTransform.
 */
export declare type PValue<T> = void | Root | PCollection<T> | PValue<T>[] | {
    [key: string]: PValue<T>;
};
/**
 * Returns a PValue as a flat object with string keys and PCollection values.
 *
 * The full set of PCollections reachable by this PValue will be returned,
 * with keys corresponding roughly to the path taken to get there.
 */
export declare function flattenPValue<T>(pValue: PValue<T>, prefix?: string): {
    [key: string]: PCollection<T>;
};
/**
 * Wraps a PValue in a single object such that a transform can be applied to it.
 *
 * For example, Flatten takes a PCollection[] as input, but Array has no
 * apply(PTransformClass) method, so one writes
 *
 *    P([pcA, pcB, pcC]).apply(new Flatten())
 */
export declare function P<T extends PValue<any>>(pvalue: T): PValueWrapper<T>;
declare class PValueWrapper<T extends PValue<any>> {
    private pvalue;
    constructor(pvalue: T);
    apply<O extends PValue<any>>(transform: PTransform<T, O>, root?: Root | null): O;
    applyAsync<O extends PValue<any>>(transform: AsyncPTransform<T, O>, root?: Root | null): Promise<O>;
    private pipeline;
}
export {};
