import { KV } from "../values";
import { PTransform, PTransformClass } from "./transform";
import { PCollection } from "../pvalue";
import { Coder } from "../coders/coders";
/**
 * An interface for performing possibly distributed, commutative, associative
 * combines (such as sum) to a set of values
 * (e.g. in `groupBy(...).combining(...)`.
 *
 * Several implementations (such as summation) are provided in the combiners
 * module.
 *
 * See also https://beam.apache.org/documentation/programming-guide/#transforms
 */
export interface CombineFn<I, A, O> {
    createAccumulator: () => A;
    addInput: (A: any, I: any) => A;
    mergeAccumulators: (accumulators: Iterable<A>) => A;
    extractOutput: (A: any) => O;
    accumulatorCoder?(inputCoder: Coder<I>): Coder<A>;
}
declare type Combiner<I> = CombineFn<I, any, any> | ((a: any, b: any) => any);
/**
 * A PTransformClass that takes a PCollection of elements, and returns a PCollection
 * of elements grouped by a field, multiple fields, an expression that is used
 * as the grouping key.
 *
 * @extends PTransformClass
 */
export declare class GroupBy<T, K> extends PTransformClass<PCollection<T>, PCollection<KV<K, Iterable<T>>>> {
    keyFn: (element: T) => K;
    keyNames: string | string[];
    keyName: string;
    /**
     * Create a GroupBy transform.
     *
     * @param key: The name of the key in the JSON object, or a function that returns the key for a given element.
     */
    constructor(key: string | string[] | ((element: T) => K), keyName?: string | undefined);
    /** @internal */
    expand(input: PCollection<T>): PCollection<KV<K, Iterable<T>>>;
    combining<I>(expr: string | ((element: T) => I), combiner: Combiner<I>, resultName: string): GroupByAndCombine<T, unknown>;
}
/**
 * Returns a PTransform that takes a PCollection of elements, and returns a
 * PCollection of elements grouped by a field, multiple fields, an expression
 * that is used as the grouping key.
 *
 * Various fields may be further aggregated with `CombineFns` by invoking
 * `groupBy(...).combining(...)`.
 */
export declare function groupBy<T, K>(key: string | string[] | ((element: T) => K), keyName?: string | undefined): GroupBy<T, K>;
/**
 * Groups all elements of the input PCollection together.
 *
 * This is generally used with one or more combining specifications, as one
 * loses parallelization benefits in bringing all elements of a distributed
 * PCollection together on a single machine.
 */
export declare class GroupGlobally<T> extends PTransformClass<PCollection<T>, PCollection<Iterable<T>>> {
    constructor();
    /** @internal */
    expand(input: any): any;
    combining<I>(expr: string | ((element: T) => I), combiner: Combiner<I>, resultName: string): GroupByAndCombine<unknown, unknown>;
}
/**
 * Returns a PTransform grouping all elements of the input PCollection together.
 *
 * This is generally used with one or more combining specifications, as one
 * loses parallelization benefits in bringing all elements of a distributed
 * PCollection together on a single machine.
 *
 * Various fields may be further aggregated with `CombineFns` by invoking
 * `groupGlobally(...).combining(...)`.
 */
export declare function groupGlobally<T>(): GroupGlobally<T>;
declare class GroupByAndCombine<T, O> extends PTransformClass<PCollection<T>, PCollection<O>> {
    keyFn: (element: T) => any;
    keyNames: string | string[] | undefined;
    combiners: CombineSpec<T, any, any>[];
    constructor(keyFn: (element: T) => any, keyNames: string | string[] | undefined, combiners: CombineSpec<T, any, any>[]);
    combining<I, O>(expr: string | ((element: T) => I), combiner: Combiner<I>, resultName: string): GroupByAndCombine<T, unknown>;
    expand(input: PCollection<T>): any;
}
export declare function countPerElement<T>(): PTransform<PCollection<T>, PCollection<{
    element: T;
    count: number;
}>>;
export declare function countGlobally<T>(): PTransform<PCollection<T>, PCollection<number>>;
interface CombineSpec<T, I, O> {
    expr: (T: any) => I;
    combineFn: CombineFn<I, any, O>;
    resultName: string;
}
/**
 * Creates a CombineFn<I, ..., I> out of a binary operator (which must be
 * commutative and associative).
 */
export declare function binaryCombineFn<I>(combiner: (a: I, b: I) => I): CombineFn<I, I | undefined, I>;
export declare function coGroupBy<T, K>(key: string | string[] | ((element: T) => K), keyName?: string | undefined): PTransform<{
    [key: string]: PCollection<any>;
}, PCollection<{
    key: K;
    values: {
        [key: string]: Iterable<any>;
    };
}>>;
export {};
