import * as runnerApi from "../proto/beam_runner_api";
import { PValue } from "../pvalue";
import { Pipeline } from "../internal/pipeline";
/**
 * Provides a custom name for an object, to be used in naming operations
 * in the pipeline.  E.g. one could write
 *
 *```js
 * pcoll.map(withName("doubleIt", (x) => (2 * x)))
 *```
 *
 * and this step in the graph will be henceforth referred to as "doubleIt."
 */
export declare function withName<T>(name: string | (() => string), arg: T): T;
/**
 * Attempts to find a suitable, human-readable name for the given object.
 *
 * If `withName` was called on this object, or it has a `beamName` member,
 * that is what is returned, otherwise it will attempt to construct a
 * nice, compact stringified representation of this object or fail with an
 * error.
 */
export declare function extractName<T>(withName: T): string;
/** @internal */
export declare class AsyncPTransformClass<InputT extends PValue<any>, OutputT extends PValue<any>> {
    beamName: string | (() => string);
    constructor(name?: string | (() => string) | null);
    expandAsync(input: InputT): Promise<OutputT>;
    expandInternalAsync(input: InputT, pipeline: Pipeline, transformProto: runnerApi.PTransform): Promise<OutputT>;
}
/** @internal */
export declare class PTransformClass<InputT extends PValue<any>, OutputT extends PValue<any>> extends AsyncPTransformClass<InputT, OutputT> {
    expand(input: InputT): OutputT;
    expandAsync(input: InputT): Promise<OutputT>;
    expandInternal(input: InputT, pipeline: Pipeline, transformProto: runnerApi.PTransform): OutputT;
    expandInternalAsync(input: InputT, pipeline: Pipeline, transformProto: runnerApi.PTransform): Promise<OutputT>;
}
/**
 * An AsyncPTransform is simply a PTransform that returns a Promise rather
 * than a concrete result.  This is required when the construction itself needs
 * to invoke asynchronous methods, and the returned promise must be awaited
 * (or chained) before further construction.
 */
export declare type AsyncPTransform<InputT extends PValue<any>, OutputT extends PValue<any>> = AsyncPTransformClass<InputT, OutputT> | ((input: InputT) => Promise<OutputT>) | ((input: InputT, pipeline: Pipeline, transformProto: runnerApi.PTransform) => Promise<OutputT>);
/**
 * A PTransform transforms an input PValue (such as a `PCollection` or set
 * thereof) into an output PValue (which may consist of zero or more
 * `PCollections`).  It is generally invoked by calling the `apply` method
 * on the input PValue, e.g.
 *
 *```js
 * const PCollection<T> input_pcoll = ...
 * const PCollection<O> output_pcoll = input_pcoll.apply(
 *     MyPTransform<PCollection<T>>, PCollection<O>>);
 *```
 *
 * Any function from the input PValue type to the output PValueType is
 * considered a valid PTransform, e.g.
 *
 *```js
 * const PCollection<T> input_pcoll = ...
 * const PCollection<O> output_pcoll = input_pcoll.apply(
 *     new function(input: PCollection<T>>): PCollection<O>> {...});
 *```
 *
 * See also the documentation on PTransforms at
 * https://beam.apache.org/documentation/programming-guide/#transforms
 */
export declare type PTransform<InputT extends PValue<any>, OutputT extends PValue<any>> = PTransformClass<InputT, OutputT> | ((input: InputT) => OutputT) | ((input: InputT, pipeline: Pipeline, transformProto: runnerApi.PTransform) => OutputT);
